/*
   A little auto-calibration utility, for boards
   that support it.

   Right now, it only supports NI E series boards,
   but it should be easily portable.

   A few things need improvement here:
    - current system gets "close", but doesn't
      do any fine-tuning
    - no pre/post gain discrimination for the
      A/D zero offset.
    - should read (and use) the actual reference
      voltage value from eeprom
    - statistics would be nice, to show how good
      the calibration is.
    - doesn't check unipolar ranges
    - "alternate calibrations" would be cool--to
      accurately measure 0 in a unipolar range
    - more portable
 */

#include <stdio.h>
#include <comedilib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <getopt.h>
#include <ctype.h>
#include <math.h>


#define N_CALDACS 16

typedef struct{
	comedi_t *dev;
	
	int dac_subdev;
	int caldac_subdev;
	int ad_subdev;

	int range;

	int n_caldacs;

	int caldacs[N_CALDACS];
	int valid_caldacs[N_CALDACS];

}calib;

double read_chan(calib *cal,int adc);
void write_caldac(calib *cal,int addr,int val);
void write_dac(calib *cal,int dac,int val);

void calib_chan(calib *it,int ad_chan,double goal);


int main(int argc, char *argv[])
{
	char *fn = NULL;
	comedi_t *dev;
	int c;
	int i;

	calib cal;
	calib *it=&cal;
	

	fn = "/dev/comedi0";
	while (1) {
		c = getopt(argc, argv, "rf");
		if (c == -1)
			break;
		switch (c) {
		case 'f':
			fn = argv[optind];
			break;
		default:
			printf("bad option\n");
			exit(1);
		}
	}

	dev = comedi_open(fn);
	if (dev == NULL ) {
		perror(fn);
		exit(0);
	}


	it->dev=dev;
	it->ad_subdev=0;
	it->dac_subdev=1;
	it->caldac_subdev=5;
	it->n_caldacs=12;

	for(i=0;i<it->n_caldacs;i++){
		it->caldacs[i]=128;
		it->valid_caldacs[i]=1;
		write_caldac(it,i,it->caldacs[i]);
	}

	/* adc offset */
	it->range=0;
	calib_chan(it,0,0);
	it->range=7;
	calib_chan(it,0,0);
	calib_chan(it,0,0);

	/* adc gain */
	it->range=8;
	calib_chan(it,5,5.0);
	calib_chan(it,5,5.0);

	/* dac0 offset */
	write_dac(it,0,0);
	it->range=7;
	calib_chan(it,2,0);
	calib_chan(it,2,0);

	/* dac1 offset */
	write_dac(it,1,0);
	it->range=7;
	calib_chan(it,3,0);
	calib_chan(it,3,0);

	/* dac0 gain */
	write_dac(it,0,2048);
	it->range=7;
	calib_chan(it,6,0);

	/* dac1 gain */
	write_dac(it,1,2048);
	it->range=7;
	calib_chan(it,7,0);

	return 0;
}



void calib_chan(calib *it,int ad_chan,double goal)
{
	double v;
	double gain[N_CALDACS];
	double maxgain=0;
	int maxgaindac=-1;
	int i;

	v=read_chan(it,ad_chan);

	puts("---");
	printf("calibrating a/d chan %d\n",ad_chan);
	printf("voltage=%g goal=%g delta=%g\n",v,goal,v-goal);
	for(i=0;i<it->n_caldacs;i++)if(it->valid_caldacs[i]){
		write_caldac(it,i,it->caldacs[i]+100);
		gain[i]=(read_chan(it,ad_chan)-v)/100.0;
		write_caldac(it,i,it->caldacs[i]);
		printf("caldac[%d] gain=%g V/bit\n",i,gain[i]);
		if(maxgain<fabs(gain[i])){
			maxgain=fabs(gain[i]);
			maxgaindac=i;
		}
	}
	printf("max gain on caldac[%d]\n",maxgaindac);

	i=maxgaindac;
	it->caldacs[i]+=rint((goal-v)/gain[i]);
	write_caldac(it,i,it->caldacs[i]);

	v=read_chan(it,ad_chan);

	printf("caldac[%d]=%d\n",i,it->caldacs[i]);
	printf("delta=%g\n",v-goal);
	
	it->valid_caldacs[i]=0;

}



/* helpers */


double read_chan(calib *cal,int adc)
{
	double data;
	comedi_sv_t sv;

	if(comedi_sv_init(&sv,cal->dev,cal->ad_subdev,adc)<0)
		printf("comedi_sv_init: error\n");

	sv.range=cal->range;
	sv.aref=3;
	sv.n=100;

	comedi_sv_measure(&sv,&data);

	return data;
}

void write_caldac(calib *cal,int addr,int val)
{
	comedi_trig it;
	unsigned int chan=CR_PACK(addr,0,0);
	sampl_t data=val;

	it.subdev = cal->caldac_subdev;
	it.mode = 0;
	it.flags = 0;
	it.n_chan = 1;
	it.chanlist = &chan;
	it.data = &data;
	it.n = 1;
	it.trigsrc = 0;
	it.trigvar = 0;
	it.trigvar1 = 0;

	if(comedi_trigger(cal->dev,&it)<0){
		perror("write_caldac");
		exit(0);
	}
}

void write_dac(calib *cal,int dac,int val)
{
	comedi_trig it;
	unsigned int chan=CR_PACK(dac,0,0);
	sampl_t data=val;

	it.subdev = cal->dac_subdev;
	it.mode = 0;
	it.flags = 0;
	it.n_chan = 1;
	it.chanlist = &chan;
	it.data = &data;
	it.n = 1;
	it.trigsrc = 0;
	it.trigvar = 0;
	it.trigvar1 = 0;

	if(comedi_trigger(cal->dev,&it)<0){
		perror("write_dac");
		exit(0);
	}
}

