/*
    lib/comedi.c
    comedi library routines

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <comedi.h>
#include <string.h>

#include <libinternal.h>


comedi_t *comedi_open(char *fn)
{
	comedi_t *it;

	if(!(it=malloc(sizeof(comedi_t))))
		goto cleanup;
	memset(it,0,sizeof(comedi_t));

	if((it->fd=open(fn,O_RDWR))<0)
		goto cleanup;

	if(ioctl_devinfo(it->fd,&it->devinfo)<0)
		goto cleanup;

	it->n_subdevices=it->devinfo.n_subdevs;

	get_subdevices(it);

	it->magic=COMEDILIB_MAGIC;

	return it;
cleanup:
	if(it)
		free(it);

	return NULL;
}

#if 0
/* this is an example of how we do versioned symbols */
__asm__(".symver comedi_open_0,comedi_open@");
#endif

void comedi_close(comedi_t *it)
{
	it->magic=0;

	/* XXX should free all memory */

	close(it->fd);
}

int comedi_fileno(comedi_t *it)
{
	return it->fd;
}

int comedi_trigger(comedi_t *it,comedi_trig *t)
{
	return ioctl_trigger(it->fd,t);
}

int __comedi_errno=0;

int comedi_errno(void)
{
	return __comedi_errno;
}

char *comedi_strerror(int errnum)
{
	return "no error";
}

void comedi_perror(const char *s)
{
	
}

int comedi_get_n_subdevices(comedi_t *it)
{
	return it->n_subdevices;
}

int comedi_get_version_code(comedi_t *it)
{
	return it->devinfo.version_code;
}

char *comedi_get_driver_name(comedi_t *it)
{
	return it->devinfo.driver_name;
}

char *comedi_get_board_name(comedi_t *it)
{
	return it->devinfo.board_name;
}

int comedi_get_subdevice_type(comedi_t *it,unsigned int subd)
{
	return it->subdevices[subd].type;
}

int comedi_get_n_channels(comedi_t *it,unsigned int subd)
{
	return it->subdevices[subd].n_chan;
}


/* */

int sc_isvalid(comedi_t *it,unsigned int subdevice,int chan)
{
	if(it->magic != COMEDILIB_MAGIC)
		return 0;
	if(subdevice>=it->n_subdevices)
		return 0;
	if(chan>=it->subdevices[subdevice].n_chan)
		return 0;

	return 1;
}

int comedi_get_maxdata(comedi_t *it,unsigned int subdevice,unsigned int chan)
{
	if(!sc_isvalid(it,subdevice,chan))
		return -1;

	if(it->subdevices[subdevice].maxdata_list)
		return it->subdevices[subdevice].maxdata_list[chan];

	return it->subdevices[subdevice].maxdata;
}

int comedi_get_rangetype(comedi_t *it,unsigned int subdevice,unsigned int chan)
{
	if(!sc_isvalid(it,subdevice,chan))
		return -1;

	if(it->subdevices[subdevice].range_type_list)
		return it->subdevices[subdevice].range_type_list[chan];

	return it->subdevices[subdevice].range_type;
}


comedi_range * comedi_get_range(comedi_t *it,unsigned int subdevice,unsigned int chan,unsigned int range)
{
	int range_type;

	if(!sc_isvalid(it,subdevice,chan))
		return NULL;

	range_type=comedi_get_rangetype(it,subdevice,chan);

	if(range>=RANGE_LENGTH(range_type))
		return NULL;

	if(it->subdevices[subdevice].rangeinfo_list)
		return it->subdevices[subdevice].rangeinfo_list[chan]+range;

	return it->subdevices[subdevice].rangeinfo+range;
}


/* */

static comedi_trig data_trig={
	mode:		0,
	flags:		0,
	n_chan:		1,
	n:		1,
	trigsrc:	0,
	trigvar:	0,
	trigvar1:	0,
};

int comedi_data_write(comedi_t *it,unsigned int subdev,unsigned int chan,unsigned int range,
		unsigned int aref,lsampl_t data)
{
	sampl_t sdata=data;

	chan=CR_PACK(chan,range,aref);

	data_trig.subdev=subdev;
	if(it->subdevices[subdev].subd_flags & SDF_LSAMPL){
		data_trig.data=(sampl_t *)(&data);
	}else{
		data_trig.data=&sdata;
	}
	data_trig.chanlist=&chan;

	return ioctl_trigger(it->fd,&data_trig);
}


int comedi_data_read(comedi_t *it,unsigned int subdev,unsigned int chan,unsigned int range,
		unsigned int aref,lsampl_t *data)
{
	int ret;
	sampl_t sdata;

	chan=CR_PACK(chan,range,aref);
	
	data_trig.subdev=subdev;
	data_trig.chanlist=&chan;
	if(it->subdevices[subdev].subd_flags & SDF_LSAMPL){
		data_trig.data=(sampl_t *)data;
	}else{
		data_trig.data=&sdata;
	}

	ret=ioctl_trigger(it->fd,&data_trig);
	if(ret<0)
		return ret;

	if(!(it->subdevices[subdev].subd_flags & SDF_LSAMPL)){
		*data=sdata;
	}

	return 0;
}


